@@ -35,30 +35,47 @@
  * \date May-Sep 2021
  */
 
-// EXPERIMENTAL for testing in JackTrip v1.5.0
-// requires server and client have same FPP
-// runs ok from FPP 16 up to 1024
-// number of in / out channels should be the same
-// mono, stereo and -n3 tested fine
+// EXPERIMENTAL for testing in JackTrip v1.5.<n>
+// server and client can have different FPP (tested from FPP 16 to 1024)
+// stress tested by repeatedly starting & stopping across range of FPP's
+// server and client can have different in / out channel count
+// large FPP, for example 512, should not be run with --udprt as PLC audio callback
+// completion gets delayed auto mode -- use -q auto3 or for manual setting of initial
+// mMsecTolerance -- use -q auto<msec> gathers data for 6 sec and then goes full auto
 
-// ./jacktrip -S --udprt -p1 --bufstrategy 3  -I 1 -q10
-// PIPEWIRE_LATENCY=32/48000 ./jacktrip -C <SERV> --udprt --bufstrategy 3 -I 1 -q4
+// example WAN test
+// ./jacktrip -S --udprt -p1 --bufstrategy 3 -q auto
+// PIPEWIRE_LATENCY=32/48000 ./jacktrip -C <SERV> --udprt --bufstrategy 3 -q auto
 
+// (mono : mono : mono)
+// ./jacktrip -S -p1 --bufstrategy 3 -q auto3 -u --receivechannels 1 --sendchannels 1
+// --udprt  -I 1
+// ./jacktrip -C <SERV> --receivechannels 1 -u --sendchannels 1 --bufstrategy 3 -q auto3
+// -I 1 --udprt
+
+// latest (mono <: stereo : stereo)
+// ./jacktrip -S -p1 --bufstrategy 3 -q auto3 -u --receivechannels 1 --sendchannels 2
+// --udprt  -I 1
+// ./jacktrip -C <SERV> --receivechannels 2 -u --sendchannels 1 --bufstrategy 3 -q auto3
+// -I 1 --udprt
+
+// example WAN test
 // at 48000 / 32 = 2.667 ms total roundtrip latency
-// local loopback test with 4 terminals running and the following jmess file
+// local loopback test with 4 terminals running and a jmess file
 // jacktrip -S --udprt --nojackportsconnect -q1 --bufstrategy 3
 // jacktrip -C localhost --udprt --nojackportsconnect -q1  --bufstrategy 3
-// use jack_iodelay
-// use jmess -s delay.xml and jmess -c delay.xml
 
 // tested outgoing loss impairments with (replace lo with relevant network interface)
-// sudo tc qdisc add dev lo root netem loss 2%
-// sudo tc qdisc del dev lo root netem loss 2%
+// sudo tc qdisc add dev lo root netem loss 5%
+// sudo tc qdisc del dev lo root netem loss 5%
+// or very revealing
+// sudo tc qdisc add dev lo root netem loss 20%
+// sudo tc qdisc del dev lo root netem loss 20%
 // tested jitter impairments with
-// for wifi
+// wifi simulation
 // sudo tc qdisc add dev lo root netem slot distribution pareto 0.1ms 3.0ms
 // sudo tc qdisc del dev lo root netem slot distribution pareto 0.1ms 3.0ms
-// for wired cmn9
+// ugly wired simulation
 // sudo tc qdisc add dev lo root netem slot distribution pareto 0.2ms 0.3ms
 // sudo tc qdisc del dev lo root netem slot distribution pareto 0.2ms 0.3ms
 
@@ -67,30 +84,49 @@
 #include <iomanip>
 #include <sstream>
 
+#include "JitterBuffer.h"
 #include "jacktrip_globals.h"
 using std::cout;
 using std::endl;
 using std::setw;
 
-// constants... tested for now
-constexpr int HIST          = 6;    // at FPP32
+// constants...
+constexpr int HIST          = 4;    // at FPP 16-128, see below for > 128
 constexpr int ModSeqNumInit = 256;  // bounds on seqnums, 65536 is max in packet header
 constexpr int NumSlotsMax   = 128;  // mNumSlots looped for recent arrivals
 constexpr int LostWindowMax = 32;   // mLostWindow looped for recent arrivals
+constexpr double DefaultAutoHeadroom =
+    3.0;                           // msec padding for auto adjusting mMsecTolerance
+constexpr double AutoMax = 250.0;  // msec bounds on insane IPI, like ethernet unplugged
+constexpr double AutoInitDur = 6000.0;  // kick in auto after this many msec
+constexpr double AutoInitValFactor =
+    0.5;  // scale for initial mMsecTolerance during init phase if unspecified
+// tweak
+constexpr int WindowDivisor = 8;     // for faster auto tracking
+constexpr int MaxFPP        = 1024;  // tested up to this FPP
 //*******************************************************************************
-Regulator::Regulator(int sample_rate, int channels, int bit_res, int FPP, int qLen)
+Regulator::Regulator(int rcvChannels, int bit_res, int FPP, int qLen, int bqLen)
     : RingBuffer(0, 0)
-    , mNumChannels(channels)
+    , mNumChannels(rcvChannels)
     , mAudioBitRes(bit_res)
     , mFPP(FPP)
-    , mSampleRate(sample_rate)
-    , mMsecTolerance((double)qLen)
+    , mMsecTolerance((double)qLen)  // handle non-auto mode, expects positive qLen
     , mAuto(false)
+    , m_b_BroadcastQueueLength(bqLen)
 {
-    if (mMsecTolerance < 0.0) {  // handle, for example, CLI -q auto15 or -q auto
-        mAuto = true;
-        mMsecTolerance *= -1.0;
-    };
+    // catch settings that are compute bound using long HIST
+    // hub client rcvChannels is set from client's settings parameters
+    // hub server rcvChannels is set from connecting client, not from hub parameters
+    //    if (mNumChannels > MaxChans) {
+    //        std::cerr << "*** Regulator.cpp: receive channels = " << mNumChannels
+    //                  << " larger than max channels = " << MaxChans << "\n";
+    //        exit(1);
+    //    }
+    if (mFPP > MaxFPP) {
+        std::cerr << "*** Regulator.cpp: local FPP = " << mFPP
+                  << " larger than max FPP = " << MaxFPP << "\n";
+        exit(1);
+    }
     switch (mAudioBitRes) {  // int from JitterBuffer to AudioInterface enum
     case 1:
         mBitResolutionMode = AudioInterface::audioBitResolutionT::BIT8;
@@ -105,13 +141,14 @@ Regulator::Regulator(int sample_rate, int channels, int bit_res, int FPP, int qL
         mBitResolutionMode = AudioInterface::audioBitResolutionT::BIT32;
         break;
     }
-    mHist            = HIST * 32;             // samples, from original settings
-    double histFloat = mHist / (double)mFPP;  // packets for other FPP
-    mHist            = (int)histFloat;
+    mHist = HIST;  // at FPP 32
+    if ((mNumChannels > 1) && (mFPP > 64))
+        mHist = 3;
+    if (mFPP > 128)
+        mHist = 2;
     if (mHist < 2)
         mHist = 2;  // min packets for prediction, needs at least 2
-    else if (mHist > 6)
-        mHist = 6;  // max packets, keep a lid on CPU load
+
     if (gVerboseFlag)
         cout << "mHist = " << mHist << " at " << mFPP << "\n";
     mBytes     = mFPP * mNumChannels * mBitResolutionMode;
@@ -124,10 +161,7 @@ Regulator::Regulator(int sample_rate, int channels, int bit_res, int FPP, int qL
         mFadeDown[i] = 1.0 - mFadeUp[i];
     }
     mLastWasGlitch = false;
-    mPacketDurMsec = 1000.0 * (double)mFPP / (double)mSampleRate;
-    if (mMsecTolerance < mPacketDurMsec)
-        mMsecTolerance = mPacketDurMsec;  // absolute minimum
-    mNumSlots = NumSlotsMax;  //((int)ceil(mMsecTolerance / mPacketDurMsec)) + PADSLOTS;
+    mNumSlots      = NumSlotsMax;
 
     for (int i = 0; i < mNumSlots; i++) {
         int8_t* tmp = new int8_t[mBytes];
@@ -143,8 +177,6 @@ Regulator::Regulator(int sample_rate, int channels, int bit_res, int FPP, int qL
     memcpy(mZeros, mXfrBuffer, mBytes);
     mAssembledPacket = new int8_t[mBytes];  // for asym
     memcpy(mAssembledPacket, mXfrBuffer, mBytes);
-    pushStat       = new StdDev(&mIncomingTimer, (int)(floor(48000.0 / (double)mFPP)), 1);
-    pullStat       = new StdDev(&mIncomingTimer, (int)(floor(48000.0 / (double)mFPP)), 2);
     mLastLostCount = 0;  // for stats
     mIncomingTimer.start();
     mLastSeqNumIn  = -1;
@@ -156,23 +188,27 @@ Regulator::Regulator(int sample_rate, int channels, int bit_res, int FPP, int qL
     mModSeqNum           = mNumSlots * 2;
     mFPPratioNumerator   = 1;
     mFPPratioDenominator = 1;
-    mPartialPacketCnt    = 0;
     mFPPratioIsSet       = false;
     mBytesPeerPacket     = mBytes;
-#ifdef GUIBS3
-    // hg for GUI
-    hg = new HerlperGUI(qApp->activeWindow());
-    connect(hg, SIGNAL(moved(double)), this, SLOT(changeGlobal(double)));
-    connect(hg, SIGNAL(moved_2(int)), this, SLOT(changeGlobal_2(int)));
-    connect(hg, SIGNAL(moved_3(int)), this, SLOT(changeGlobal_3(int)));
-#endif
+    mAssemblyCnt         = 0;
+    mModCycle            = 1;
+    mModSeqNumPeer       = 1;
+    mPeerFPP             = mFPP;  // use local until first packet arrives
+    mAutoHeadroom        = DefaultAutoHeadroom;
+    mFPPdurMsec          = 1000.0 * mFPP / 48000.0;
     changeGlobal_3(LostWindowMax);
     changeGlobal_2(NumSlotsMax);  // need hg if running GUI
-    changeGlobal((double)qLen);
+    if (m_b_BroadcastQueueLength) {
+        m_b_ReceiveRingBuffer = new JitterBuffer(
+            mFPP, qLen, 48000, 1, m_b_BroadcastQueueLength, mNumChannels, mAudioBitRes);
+        qDebug() << "Broadcast started in Regulator with packet queue of"
+                 << m_b_BroadcastQueueLength;
+        // have not implemented the mJackTrip->queueLengthChanged functionality
+    }
 }
 
 void Regulator::changeGlobal(double x)
-{  // mMsecTolerance
+{
     mMsecTolerance = x;
     printParams();
 }
@@ -194,75 +230,112 @@ void Regulator::changeGlobal_3(int x)
     printParams();
 }
 
-void Regulator::printParams()
-{
-//    qDebug() << "mMsecTolerance" << mMsecTolerance << "mNumSlots" << mNumSlots
-//             << "mModSeqNum" << mModSeqNum << "mLostWindow" << mLostWindow;
-#ifdef GUIBS3
-    updateGUI((int)mMsecTolerance, mNumSlots);
-#endif
+void Regulator::printParams(){
+    //    qDebug() << "mMsecTolerance" << mMsecTolerance << "mNumSlots" << mNumSlots
+    //             << "mModSeqNum" << mModSeqNum << "mLostWindow" << mLostWindow;
 };
 
-#ifdef GUIBS3
-void Regulator::updateGUI(double msTol, int nSlots)
-{
-    hg->updateDisplay(msTol, nSlots, 0);  // need to remove last param
-}
-#endif
-
 Regulator::~Regulator()
 {
-    delete mXfrBuffer;
-    delete mZeros;
+    delete[] mXfrBuffer;
+    delete[] mZeros;
+    delete[] mAssembledPacket;
+    delete pushStat;
+    delete pullStat;
     for (int i = 0; i < mNumChannels; i++)
         delete mChanData[i];
+    for (auto& slot : mSlots) {
+        delete[] slot;
+    };
+    if (m_b_BroadcastQueueLength)
+        delete m_b_ReceiveRingBuffer;
 }
 
-void Regulator::setFPPratio(int len)
+void Regulator::setFPPratio()
 {
-    int peerFPP = len / (mNumChannels * mBitResolutionMode);
-    if (peerFPP != mFPP) {
-        if (peerFPP > mFPP)
-            mFPPratioDenominator = peerFPP / mFPP;
+    if (mPeerFPP != mFPP) {
+        if (mPeerFPP > mFPP)
+            mFPPratioDenominator = mPeerFPP / mFPP;
         else
-            mFPPratioNumerator = mFPP / peerFPP;
-        qDebug() << "peerBuffers / localBuffers" << mFPPratioNumerator << " / "
-                 << mFPPratioDenominator;
+            mFPPratioNumerator = mFPP / mPeerFPP;
+        //        qDebug() << "peerBuffers / localBuffers" << mFPPratioNumerator << " / "
+        //                 << mFPPratioDenominator;
     }
-    if (mFPPratioNumerator > 1)
+    if (mFPPratioNumerator > 1) {
         mBytesPeerPacket = mBytes / mFPPratioNumerator;
-    mFPPratioIsSet = true;
+        mModCycle        = mFPPratioNumerator - 1;
+        mModSeqNumPeer   = mModSeqNum * mFPPratioNumerator;
+    } else if (mFPPratioDenominator > 1) {
+        mModSeqNumPeer = mModSeqNum / mFPPratioDenominator;
+    }
 }
 
 //*******************************************************************************
 void Regulator::shimFPP(const int8_t* buf, int len, int seq_num)
 {
     if (seq_num != -1) {
-        if (!mFPPratioIsSet)
-            setFPPratio(len);
-        if (mFPPratioNumerator > 1) {  // 2/1, 4/1 peer FPP is lower
-            int modSeqNumPeer = mModSeqNum * mFPPratioNumerator;
-            seq_num %= modSeqNumPeer;
-            //        qDebug() << seq_num << seq_num / mFPPratioNumerator <<
-            //        mPartialPacketCnt;
-            seq_num /= mFPPratioNumerator;
-            int tmp = (mPartialPacketCnt % mFPPratioNumerator) * mBytesPeerPacket;
-            memcpy(&mAssembledPacket[tmp], buf, mBytesPeerPacket);
-            if ((mPartialPacketCnt % mFPPratioNumerator) == (mFPPratioNumerator - 1))
-                pushPacket(mAssembledPacket, seq_num);
-            mPartialPacketCnt++;
-        } else if (mFPPratioDenominator > 1) {  // 1/2, 1/4 peer FPP is higher
-            int modSeqNumPeer = mModSeqNum / mFPPratioDenominator;
-            seq_num %= modSeqNumPeer;
-            seq_num *= mFPPratioDenominator;
-            for (int i = 0; i < mFPPratioDenominator; i++) {
-                int tmp = i * mBytes;
-                memcpy(mAssembledPacket, &buf[tmp], mBytes);
-                pushPacket(mAssembledPacket, seq_num);
-                seq_num++;
-            }
-        } else
+        if (!mFPPratioIsSet) {  // first peer packet
+            mPeerFPP = len / (mNumChannels * mBitResolutionMode);
+            // bufstrategy 1 autoq mode overloads qLen with negative val
+            // creates this ugly code
+            if (mMsecTolerance < 0) {  // handle -q auto or, for example, -q auto10
+                mAuto = true;
+                // default is -500 from bufstrategy 1 autoq mode
+                // tweak
+                if (mMsecTolerance != -500.0) {
+                    // use it to set headroom
+                    mAutoHeadroom = -mMsecTolerance;
+                    qDebug() << "PLC is in auto mode and has been set with"
+                             << mAutoHeadroom << "ms headroom";
+                    if (mAutoHeadroom > 50.0)
+                        qDebug() << "That's a very large value and should be less than, "
+                                    "for example, 50ms";
+                }
+                // found an interesting relationship between mPeerFPP and initial
+                // mMsecTolerance mPeerFPP*0.5 is pretty good though that's an oddball
+                // conversion of bufsize directly to msec
+                mMsecTolerance = (mPeerFPP * AutoInitValFactor);
+            };
+            setFPPratio();
+            // number of stats tick calls per sec depends on FPP
+            int maxFPP = (mPeerFPP > mFPP) ? mPeerFPP : mFPP;
+            pushStat =
+                new StdDev(1, &mIncomingTimer, (int)(floor(48000.0 / (double)maxFPP)));
+            pullStat =
+                new StdDev(2, &mIncomingTimer, (int)(floor(48000.0 / (double)mFPP)));
+            mFPPratioIsSet = true;
+        }
+        if (mFPPratioNumerator == mFPPratioDenominator) {
             pushPacket(buf, seq_num);
+        } else {
+            seq_num %= mModSeqNumPeer;
+            if (mFPPratioNumerator > 1) {  // 2/1, 4/1 peer FPP is lower, , (local/peer)/1
+                int tmp = (seq_num % mFPPratioNumerator) * mBytesPeerPacket;
+                memcpy(&mAssembledPacket[tmp], buf, mBytesPeerPacket);
+                if ((seq_num % mFPPratioNumerator) == mModCycle) {
+                    if (mAssemblyCnt == mModCycle)
+                        pushPacket(mAssembledPacket, seq_num / mFPPratioNumerator);
+                    //                    else
+                    //                        qDebug() << "incomplete due to lost packet";
+                    mAssemblyCnt = 0;
+                } else
+                    mAssemblyCnt++;
+            } else if (mFPPratioDenominator
+                       > 1) {  // 1/2, 1/4 peer FPP is higher, 1/(peer/local)
+                seq_num *= mFPPratioDenominator;
+                for (int i = 0; i < mFPPratioDenominator; i++) {
+                    int tmp = i * mBytes;
+                    memcpy(mAssembledPacket, &buf[tmp], mBytes);
+                    pushPacket(mAssembledPacket, seq_num);
+                    seq_num++;
+                }
+            }
+        }
+        pushStat->tick();
+        double adjustAuto = pushStat->calcAuto(mAutoHeadroom, mFPPdurMsec);
+        //        qDebug() << adjustAuto;
+        if (mAuto && (pushStat->lastTime > AutoInitDur))
+            mMsecTolerance = adjustAuto;
     }
 };
 
@@ -270,20 +343,13 @@ void Regulator::shimFPP(const int8_t* buf, int len, int seq_num)
 void Regulator::pushPacket(const int8_t* buf, int seq_num)
 {
     QMutexLocker locker(&mMutex);
-    //    qDebug() << "\t" << seq_num;
     seq_num %= mModSeqNum;
-    // if (seq_num==0) return;   // if (seq_num==1) return; // impose regular loss
+    // if (seq_num==0) return;   // impose regular loss
     mIncomingTiming[seq_num] =
         mMsecTolerance + (double)mIncomingTimer.nsecsElapsed() / 1000000.0;
     mLastSeqNumIn = seq_num;
     if (mLastSeqNumIn != -1)
         memcpy(mSlots[mLastSeqNumIn % mNumSlots], buf, mBytes);
-    double nowMS = pushStat->tick();
-    if (mAuto && (nowMS > 2000.0)) {
-        double tmp = pushStat->longTermStdDev + pushStat->longTermMax;
-        tmp += 2.0;  // 2 ms -- kind of a guess
-        changeGlobal(tmp);
-    }
 };
 
 //*******************************************************************************
@@ -291,7 +357,7 @@ void Regulator::pullPacket(int8_t* buf)
 {
     QMutexLocker locker(&mMutex);
     mSkip = 0;
-    if (mLastSeqNumIn == -1) {
+    if ((mLastSeqNumIn == -1) || (!mFPPratioIsSet)) {
         goto ZERO_OUTPUT;
     } else {
         mLastSeqNumOut++;
@@ -316,16 +382,19 @@ void Regulator::pullPacket(int8_t* buf)
     }
 
 PACKETOK : {
-    if (mSkip)
+    if (mSkip) {
         processPacket(true);
-    else
+        pullStat->plcOverruns += mSkip;
+    } else
         processPacket(false);
+    pullStat->tick();
     goto OUTPUT;
 }
 
 UNDERRUN : {
     processPacket(true);
     pullStat->plcUnderruns++;  // count late
+    pullStat->tick();
     goto OUTPUT;
 }
 
@@ -334,17 +403,28 @@ ZERO_OUTPUT:
 
 OUTPUT:
     memcpy(buf, mXfrBuffer, mBytes);
-    pullStat->tick();
 };
 
 //*******************************************************************************
 void Regulator::processPacket(bool glitch)
 {
+    double tmp = 0.0;
+    if ((glitch) && (mFPPratioDenominator > 1)) {
+        glitch = !(mLastSeqNumOut % mFPPratioDenominator);
+    }
+    if (glitch)
+        tmp = (double)mIncomingTimer.nsecsElapsed();
     for (int ch = 0; ch < mNumChannels; ch++)
         processChannel(ch, glitch, mPacketCnt, mLastWasGlitch);
     mLastWasGlitch = glitch;
     mPacketCnt++;
     // 32 bit is good for days:  (/ (* (- (expt 2 32) 1) (/ 32 48000.0)) (* 60 60 24))
+
+    if (glitch) {
+        double tmp2 = (double)mIncomingTimer.nsecsElapsed() - tmp;
+        tmp2 /= 1000000.0;
+        pullStat->lastPLCdspElapsed = tmp2;
+    }
 }
 
 //*******************************************************************************
@@ -367,9 +447,10 @@ void Regulator::processChannel(int ch, bool glitch, int packetCnt, bool lastWasG
             // LINEAR PREDICT DATA
             cd->mTail = cd->mTrain;
 
-            ba.predict(cd->mCoeffs, cd->mTail);  // resizes to TRAINSAMPS-2 + TRAINSAMPS
+            ba.predict(cd->mCoeffs,
+                       cd->mTail);  // resizes to TRAINSAMPS-2 + TRAINSAMPS
 
-            for (int i = 0; i < (cd->trainSamps - 1); i++)
+            for (int i = 0; i < (cd->trainSamps - 2); i++)
                 cd->mPrediction[i] = cd->mTail[i + cd->trainSamps];
         }
         // cross fade last prediction with mTruth
@@ -450,7 +531,7 @@ bool BurgAlgorithm::classify(double d)
         tmp = true;
         break;
     case FP_ZERO:
-        //      qDebug() <<  ("zero");
+        qDebug() << ("zero");
         tmp = true;
         break;
     case FP_SUBNORMAL:
@@ -470,8 +551,8 @@ void BurgAlgorithm::train(std::vector<long double>& coeffs, const std::vector<fl
     size_t N = x.size() - 1;
     size_t m = coeffs.size();
 
-    //        if (x.size() < m) qDebug() << "time_series should have more elements than
-    //        the AR order is";
+    //        if (x.size() < m) qDebug() << "time_series should have more elements
+    //        than the AR order is";
 
     // INITIALIZE Ak
     //    vector<long double> Ak(m + 1, 0.0);
@@ -575,8 +656,9 @@ ChanData::ChanData(int i, int FPP, int hist) : ch(i)
 }
 
 //*******************************************************************************
-StdDev::StdDev(QElapsedTimer* timer, int w, int id) : mTimer(timer), window(w), mId(id)
+StdDev::StdDev(int id, QElapsedTimer* timer, int w) : mId(id), mTimer(timer), window(w)
 {
+    window /= WindowDivisor;
     reset();
     longTermStdDev    = 0.0;
     longTermStdDevAcc = 0.0;
@@ -587,21 +669,35 @@ StdDev::StdDev(QElapsedTimer* timer, int w, int id) : mTimer(timer), window(w),
     longTermMax       = 0.0;
     longTermMaxAcc    = 0.0;
     lastTime          = 0.0;
+    lastPLCdspElapsed = 0.0;
     data.resize(w, 0.0);
 }
 
 void StdDev::reset()
 {
-    mean = 0.0;
-    //        varRunning = 0.0;
+    ctr          = 0;
+    plcOverruns  = 0;
+    plcUnderruns = 0;
+    mean         = 0.0;
     acc          = 0.0;
     min          = 999999.0;
-    max          = 0.0;
-    ctr          = 0;
-    plcUnderruns = 0;
+    max          = -999999.0;
 };
 
-double StdDev::tick()
+double StdDev::calcAuto(double autoHeadroom, double localFPPdur)
+{
+    //    qDebug() << longTermStdDev << longTermMax << AutoMax << window <<
+    //    longTermCnt;
+    if ((longTermStdDev == 0.0) || (longTermMax == 0.0))
+        return AutoMax;
+    double tmp = longTermStdDev + ((longTermMax > AutoMax) ? AutoMax : longTermMax);
+    if (tmp < localFPPdur)
+        tmp = localFPPdur;  // might also check peerFPP...
+    tmp += autoHeadroom;
+    return tmp;
+};
+
+void StdDev::tick()
 {
     double now       = (double)mTimer->nsecsElapsed() / 1000000.0;
     double msElapsed = now - lastTime;
@@ -622,15 +718,15 @@ double StdDev::tick()
             var += (tmp * tmp);
         }
         var /= (double)window;
-        double stdDev = sqrt(var);
+        double stdDevTmp = sqrt(var);
         if (longTermCnt) {
-            longTermStdDevAcc += stdDev;
+            longTermStdDevAcc += stdDevTmp;
             longTermStdDev = longTermStdDevAcc / (double)longTermCnt;
             longTermMaxAcc += max;
             longTermMax = longTermMaxAcc / (double)longTermCnt;
             if (gVerboseFlag)
                 cout << setw(10) << mean << setw(10) << lastMin << setw(10) << max
-                     << setw(10) << stdDev << setw(10) << longTermStdDev << " " << mId
+                     << setw(10) << stdDevTmp << setw(10) << longTermStdDev << " " << mId
                      << endl;
         } else if (gVerboseFlag)
             cout << "printing directly from Regulator->stdDev->tick:\n (mean / min / "
@@ -638,19 +734,21 @@ double StdDev::tick()
                     "stdDev / longTermStdDev) \n";
 
         longTermCnt++;
-        lastMean   = mean;
-        lastMin    = min;
-        lastMax    = max;
-        lastStdDev = stdDev;
+        lastMean         = mean;
+        lastMin          = min;
+        lastMax          = max;
+        lastPlcOverruns  = plcOverruns;
+        lastPlcUnderruns = plcUnderruns;
+        lastStdDev       = stdDevTmp;
         reset();
     }
-    return lastTime;
 }
+
 //*******************************************************************************
 bool Regulator::getStats(RingBuffer::IOStat* stat, bool reset)
 {
     QMutexLocker locker(&mMutex);
-    if (reset) {  // all are unused
+    if (reset) {  // all are unused, this is copied from superclass
         mUnderruns        = 0;
         mOverflows        = 0;
         mSkew0            = mLevel;
@@ -661,68 +759,23 @@ bool Regulator::getStats(RingBuffer::IOStat* stat, bool reset)
         mBufIncCompensate = 0;
         mBroadcastSkew    = 0;
     }
-    // hijack  of  struct IOStat {
-    stat->underruns = pullStat->lastPlcUnderruns;
-#define FLOATFACTOR 1000.0
-    stat->overflows         = FLOATFACTOR * pushStat->longTermStdDev;
-    stat->skew              = FLOATFACTOR * pushStat->lastMean;
-    stat->skew_raw          = FLOATFACTOR * pushStat->lastMin;
-    stat->level             = FLOATFACTOR * pushStat->lastMax;
-    stat->buf_dec_overflows = FLOATFACTOR * pushStat->lastStdDev;
 
+    // hijack  of  struct IOStat {
+    stat->underruns = pullStat->lastPlcUnderruns + pullStat->lastPlcOverruns;
+#define FLOATFACTOR 1000.0
+    stat->overflows = FLOATFACTOR * pushStat->longTermStdDev;
+    stat->skew      = FLOATFACTOR * pushStat->lastMean;
+    stat->skew_raw  = FLOATFACTOR * pushStat->lastMin;
+    stat->level     = FLOATFACTOR * pushStat->lastMax;
+    //    stat->level              = FLOATFACTOR * pushStat->longTermMax;
+    stat->buf_dec_overflows  = FLOATFACTOR * pushStat->lastStdDev;
+    stat->autoq_corr         = FLOATFACTOR * mMsecTolerance;
     stat->buf_dec_pktloss    = FLOATFACTOR * pullStat->longTermStdDev;
     stat->buf_inc_underrun   = FLOATFACTOR * pullStat->lastMean;
     stat->buf_inc_compensate = FLOATFACTOR * pullStat->lastMin;
     stat->broadcast_skew     = FLOATFACTOR * pullStat->lastMax;
     stat->broadcast_delta    = FLOATFACTOR * pullStat->lastStdDev;
-    // unused
-    //        int32_t autoq_corr;
-    //        int32_t autoq_rate;
+    stat->autoq_rate         = FLOATFACTOR * pullStat->lastPLCdspElapsed;
+    // none are unused
     return true;
 }
-/*
-QString Regulator::getStats(uint32_t statCount, uint32_t lostCount)
-{
-    // formatting floats in columns looks better with std::stringstream than with
-    // QTextStream
-    QString tmp;
-    if (!statCount) {
-        tmp = QString("Regulator: inter-packet intervals msec\n");
-        tmp += "                 (window of last ";
-        tmp += QString::number(pullStat->window);
-        tmp += " packets)\n";
-        tmp +=
-                "secs   avgStdDev (mean       min       max     stdDev) "
-                "PLC(under over  skipped) lost\n";
-    } else {
-        uint32_t lost  = lostCount - mLastLostCount;
-        mLastLostCount = lostCount;
-#define PDBL(x)  << setw(10) << (QString("%1").arg(pushStat->x, 0, 'f', 2)).toStdString()
-#define PDBL2(x) << setw(10) << (QString("%1").arg(pullStat->x, 0, 'f', 2)).toStdString()
-        std::stringstream logger;
-        logger << setw(2)
-               << statCount
-                  PDBL(longTermStdDev) PDBL(lastMean) PDBL(lastMin) PDBL(lastMax)
-PDBL(lastStdDev)
-               << setw(8) << pushStat->lastPlcSkipped
-          #ifndef GUIBS3
-                  // comment out this next line for GUI because...
-               << endl
-                  // ...print all stats in one line when running in GUI because can't
-handle extra crlf
-                  // and... to actually see the two lines, need to run it in terminal
-          #endif
-                  ;
-        tmp = QString::fromStdString(logger.str());
-        std::stringstream logger2;
-        logger2 << setw(2)
-                << "" PDBL2(longTermStdDev) PDBL2(lastMean) PDBL2(lastMin) PDBL2(lastMax)
-                   PDBL2(lastStdDev)
-                << setw(8) << pullStat->lastPlcUnderruns << setw(8)
-                << pullStat->lastPlcOverruns << setw(8) << pullStat->lastPlcSkipped
-                << setw(8) << lost << endl;
-        tmp += QString::fromStdString(logger2.str());
-    }
-    return tmp;
-}
-*/
